# -*- coding: utf-8 -*-

import os

from bio2bel.utils import get_connection, get_data_dir

MODULE_NAME = "hmdd"
DATA_DIR = get_data_dir(MODULE_NAME)
DEFAULT_CACHE_CONNECTION = get_connection(MODULE_NAME)
DATA_URL = "http://www.cuilab.cn/files/images/hmdd2/alldata.txt"
DATA_FILE_PATH = os.path.join(DATA_DIR, "hmddRawData.tsv")
col_names = ["Index", "miRNA ID", "MeSHDisease term", "PubMed ID", "Description"]


# -*- coding: utf-8 -*-

from sqlalchemy import Column, ForeignKey, Integer, String
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relationship
from .constants import MODULE_NAME

from pybel.dsl import mirna as mirna_dsl, pathology as pathology_dsl
from pybel.constants import ASSOCIATION

MIRNA_TABLE_NAME = '{}_mirna'.format(MODULE_NAME)
DISEASE_TABLE_NAME = '{}_disease'.format(MODULE_NAME)
ASSOCICATION_TABLE_NAME = '{}_association'.format(MODULE_NAME)

Base = declarative_base()


class MiRNA(Base):
    """This class represents the miRNA table"""

    __tablename__ = MIRNA_TABLE_NAME
    id = Column(Integer, primary_key=True)
    name = Column(String(255), nullable=False, unique=True, index=True, doc='name from mirBase')

    def __repr__(self):
        return self.name


class Disease(Base):
    """This class represents the disease table"""

    __tablename__ = DISEASE_TABLE_NAME
    id = Column(Integer, primary_key=True)
    name = Column(String(255), nullable=False, unique=True, index=True, doc='name from MeSH')

    def __repr__(self):
        return self.name


class Association(Base):
    """This class represents the miRNA disease association table"""

    __tablename__ = ASSOCICATION_TABLE_NAME
    id = Column(Integer, primary_key=True)
    pubmed = Column(String(32), nullable=False)
    description = Column(String, doc='This is a manually curated association')
    mirna_id = Column(Integer, ForeignKey('{}.id'.format(MIRNA_TABLE_NAME)))
    mirna = relationship('MiRNA')
    disease_id = Column(Integer, ForeignKey('{}.id'.format(DISEASE_TABLE_NAME)))
    disease = relationship('Disease')

    def add_to_bel_graph(self, graph):
        """Add this association to a BEL graph

        :param pybel.BELGraph graph:
        """
        mirna_node = mirna_dsl(namespace='MIRBASE', name=self.mirna.name)
        disease_node = pathology_dsl(namespace='MESH', name=self.disease.name)

        graph.add_qualified_edge(
            mirna_node,
            disease_node,
            relation=ASSOCIATION,
            citation=str(self.pubmed),
            evidence=str(self.description),
        )



# -*- coding: utf-8 -*-

"""This module builds a :mod:`Flask` application for interacting with the underlying database. When installing,
use the web extra like:

.. source-code:: sh

    pip install bio2bel_hmdd[web]
"""

import flask_admin
from flask import Flask
from flask_admin.contrib.sqla import ModelView

from bio2bel_hmdd.manager import Manager
from bio2bel_hmdd.models import *


def add_admin(app, session, **kwargs):
    """Adds a Flask Admin interface to an application

    :param flask.Flask app:
    :param session:
    :param kwargs:
    :rtype: flask_admin.Admin
    """
    admin = flask_admin.Admin(app, **kwargs)

    admin.add_view(ModelView(MiRNA, session))
    admin.add_view(ModelView(Disease, session))
    admin.add_view(ModelView(Association, session))

    return admin


def get_app(connection=None, url=None):
    """Creates a Flask application

    :type connection: Optional[str or bio2bel_hmdd.Manager]
    :type url: Optional[str]
    :rtype: flask.Flask
    """
    app = Flask(__name__)
    manager = Manager.ensure(connection=connection)
    add_admin(app, manager.session, url=url)
    return app


if __name__ == '__main__':
    app_ = get_app(url='/')
    app_.run(debug=True, host='0.0.0.0', port=5000)



# -*- coding: utf-8 -*-

import logging

from bio2bel.utils import get_connection
from pybel import BELGraph
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from tqdm import tqdm

from .constants import MODULE_NAME
from .models import Disease, MiRNA, Association, Base
from .parser import get_hmdd_df

__all__ = ['Manager']

log = logging.getLogger(__name__)


class Manager(object):
    def __init__(self, connection=None):
        self.connection = get_connection(MODULE_NAME, connection=connection)
        self.engine = create_engine(self.connection)
        self.session_maker = sessionmaker(bind=self.engine, autoflush=False, expire_on_commit=False)
        self.session = self.session_maker()
        self.create_all()

        self.name_mirna = {}
        self.name_disease = {}

    def create_all(self, check_first=True):
        """Create the empty database (tables)"""
        Base.metadata.create_all(self.engine, checkfirst=check_first)

    def drop_all(self, check_first=True):
        """Create the empty database (tables)"""
        Base.metadata.drop_all(self.engine, checkfirst=check_first)

    @staticmethod
    def ensure(connection=None):
        """Checks and allows for a Manager to be passed to the function.

        :param connection: can be either an already build manager or a connection string to build a manager with.
        """
        if connection is None or isinstance(connection, str):
            return Manager(connection=connection)

        if isinstance(connection, Manager):
            return connection

        raise TypeError

    def get_mirna_by_name(self, name):
        """Gets an miRNA from the database if it exists

        :param str name: A mirBase name
        :rtype: Optional[MiRNA]
        """
        return self.session.query(MiRNA).filter(MiRNA.name == name).one_or_none()

    def get_disease_by_name(self, name):
        """Gets a Disease from the database if it exists

        :param str name: A MeSH disease name
        :rtype: Optional[Disease]
        """
        return self.session.query(Disease).filter(Disease.name == name).one_or_none()

    def get_or_create_mirna(self, name):
        """Gets an miRNA from the database or creates it if it does not exist

        :param str name: A mirBase name
        :rtype: MiRNA
        """
        mirna = self.name_mirna.get(name)
        if mirna is not None:
            return mirna

        mirna = self.get_mirna_by_name(name)
        if mirna is not None:
            self.name_mirna[name] = mirna
            return mirna

        mirna = self.name_mirna[name] = MiRNA(name=name)
        self.session.add(mirna)

        return mirna

    def get_or_create_disease(self, name):
        """Gets a Disease from the database or creates it if it does not exist

        :param str name: A MeSH disease name
        :rtype: Disease
        """
        disease = self.name_disease.get(name)
        if disease is not None:
            return disease

        disease = self.get_disease_by_name(name)
        if disease is not None:
            self.name_disease[name] = disease
            return disease

        disease = self.name_disease[name] = Disease(name=name)
        self.session.add(disease)

        return disease

    def _count_model(self, model):
        return self.session.query(model).count()

    def count_mirnas(self):
        """Counts the number of miRNAs in the database

        :rtype: int
        """
        return self._count_model(MiRNA)

    def count_diseases(self):
        """Counts the number of diseases in the database

        :rtype: int
        """
        return self._count_model(Disease)

    def count_associations(self):
        """Counts the number of miRNA-disease associations in the database

        :rtype: int
        """
        return self._count_model(Association)

    def populate(self, url=None):
        """Populates the database

        :param Optional[str] url: A custom data source URL
        """
        df = get_hmdd_df(url=url)

        log.info('building models')
        for _, idx, mirna_name, disease_name, pubmed, description in tqdm(df.itertuples(), total=len(df.index)):
            mirna = self.get_or_create_mirna(mirna_name)
            disease = self.get_or_create_disease(disease_name)

            association = Association(
                pubmed=pubmed,
                description=description,
                mirna=mirna,
                disease=disease,
            )

            self.session.add(association)

        log.info('inserting models')
        self.session.commit()

    def get_associations(self):
        """Returns all associations

        :rtype: list[Association]
        """
        return self.session.query(Association).all()

    def to_bel_graph(self):
        """Builds a BEL graph containing all of the miRNA-disease associations in the database

        :rtype: BELGraph
        """
        graph = BELGraph()

        for association in self.get_associations():
            association.add_to_bel_graph(graph)

        return graph






# -*- coding: utf-8 -*-

"""Bio2BEL HMDD is a package which allows the user to work with the Human microRNA Disease Database (HMDD).


Installation
------------
Get the Latest
~~~~~~~~~~~~~~~
Download the most recent code from `GitHub <https://github.com/bio2bel/hmdd>`_ with:

.. code-block:: sh

   $ python3 -m pip install git+https://github.com/bio2bel/hmdd.git

For Developers
~~~~~~~~~~~~~~
Clone the repository from `GitHub <https://github.com/bio2bel/hmdd>`_ and install in editable mode with:

.. code-block:: sh

   $ git clone https://github.com/bio2bel/hmdd.git
   $ cd hmdd
   $ python3 -m pip install -e .


Setup
-----
1. Create a :class:`bio2bel_hmdd.Manager` object
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
>>> from bio2bel_hmdd import Manager
>>> manager = Manager()

2. Populate the database
~~~~~~~~~~~~~~~~~~~~~~~~
This step will take sometime since the HMDD data needs to be downloaded, parsed, and fed into the database line
by line.

>>> manager.populate()

3. Get all associations to iterate and do magic

>>> associations = manager.get_associations()
"""

from .manager import Manager

__version__ = '0.0.1'

__title__ = 'bio2bel_hmdd'
__description__ = "A package for converting the Human microRNA Disease Database (HMDD) to BEL."
__url__ = 'https://github.com/bio2bel/hmdd'

__author__ = 'Mehdi Ali, Dejan Dukic, and Charles Tapley Hoyt'
__email__ = 'charles.hoyt@scai.fraunhofer.de'

__license__ = 'MIT License'
__copyright__ = 'Copyright (c) 2018 Mehdi Ali, Dejan Dukic, and Charles Tapley Hoyt'



# -*- coding: utf-8 -*-

import logging

import click

from .constants import DEFAULT_CACHE_CONNECTION
from .manager import Manager


@click.group()
@click.option('-c', '--connection', help='Defaults to {}'.format(DEFAULT_CACHE_CONNECTION))
@click.pass_context
def main(ctx, connection):
    """HMDB to BEL"""
    logging.basicConfig(level=logging.INFO, format="%(asctime)s - %(levelname)s - %(message)s")
    ctx.obj = Manager(connection=connection)


@main.command()
@click.pass_obj
def populate(manager):
    """Populate the database"""
    manager.populate()


@main.command()
@click.pass_obj
def drop(manager):
    """Drop the database"""
    manager.drop_all()


@main.command()
@click.option('-v', '--debug', is_flag=True)
@click.option('-p', '--port')
@click.option('-h', '--host')
@click.pass_obj
def web(manager, debug, port, host):
    """Run the web app"""
    from .web import get_app
    app = get_app(connection=manager, url='/')
    app.run(host=host, port=port, debug=debug)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

import logging
import os
from urllib.request import urlretrieve

import pandas as pd

from .constants import DATA_FILE_PATH, DATA_URL, col_names

log = logging.getLogger(__name__)


def download_hmdd(force_download=False):
    """Download the HMDD database as a TSV

    :param bool force_download: If true, forces the data to get downloaded again; defaults to False
    :return: The system file path of the downloaded file
    :rtype: str
    """
    if os.path.exists(DATA_FILE_PATH) and not force_download:
        log.info('using cached data at %s', DATA_FILE_PATH)
    else:
        log.info('downloading %s to %s', DATA_URL, DATA_FILE_PATH)
        urlretrieve(DATA_URL, DATA_FILE_PATH)

    return DATA_FILE_PATH


def get_hmdd_df(url=None, cache=True, force_download=False):
    """Loads the HMDD into a data frame

    1) Index
    2) miRNA ID
    3) MeSHDisease term
    4) PubMed ID
    5) Description

    :param Optional[str] url: A custom path to use for data
    :param bool cache: If true, the data is downloaded to the file system, else it is loaded from the internet
    :param bool force_download: If true, overwrites a previously cached file
    :rtype: pandas.DataFrame
    """
    if url is None and cache:
        url = download_hmdd(force_download=force_download)

    df = pd.read_csv(
        url or DATA_URL,
        sep='\t',
        names=col_names
    )

    return df



# -*- coding: utf-8 -*-

import codecs
import os
import re

import setuptools

PACKAGES = setuptools.find_packages(where='src')
META_PATH = os.path.join('src', 'bio2bel_hmdd', '__init__.py')
INSTALL_REQUIRES = [
    'pybel>=0.11.1',
    'click',
    'sqlalchemy',
    'bio2bel',
    'pandas',
    'tqdm'
]
EXTRAS_REQUIRE = {
    'web': ['flask', 'flask_admin'],
}
ENTRY_POINTS = {
    'bio2bel': [
        'hmdd = bio2bel_hmdd',
    ],
    'console_scripts': [
        'bio2bel_hmdd = bio2bel_hmdd.cli:main'
    ]
}

HERE = os.path.abspath(os.path.dirname(__file__))


def read(*parts):
    """Build an absolute path from *parts* and return the contents of the resulting file. Assume UTF-8 encoding."""
    with codecs.open(os.path.join(HERE, *parts), 'rb', 'utf-8') as f:
        return f.read()


META_FILE = read(META_PATH)


def find_meta(meta):
    """Extract __*meta*__ from META_FILE"""
    meta_match = re.search(
        r'^__{meta}__ = ["\']([^"\']*)["\']'.format(meta=meta),
        META_FILE, re.M
    )
    if meta_match:
        return meta_match.group(1)
    raise RuntimeError('Unable to find __{meta}__ string'.format(meta=meta))


def get_long_description():
    """Get the long_description from the README.rst file. Assume UTF-8 encoding."""
    with codecs.open(os.path.join(HERE, 'README.rst'), encoding='utf-8') as f:
        long_description = f.read()
    return long_description


if __name__ == '__main__':
    setuptools.setup(
        name=find_meta('title'),
        version=find_meta('version'),
        description=find_meta('description'),
        long_description=get_long_description(),
        url=find_meta('url'),
        author=find_meta('author'),
        author_email=find_meta('email'),
        maintainer=find_meta('author'),
        maintainer_email=find_meta('email'),
        license=find_meta('license'),
        packages=PACKAGES,
        package_dir={'': 'src'},
        install_requires=INSTALL_REQUIRES,
        extras_require=EXTRAS_REQUIRE,
        entry_points=ENTRY_POINTS,
    )
